@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 #
-# 2010-2013 Nico Schottelius (nico-cdist at schottelius.org)
+# 2010-2015 Nico Schottelius (nico-cdist at schottelius.org)
 #
 # This file is part of cdist.
 #
@@ -162,7 +162,8 @@ class Config(object):
     def object_list(self):
         """Short name for object list retrieval"""
         for cdist_object in core.CdistObject.list_objects(self.local.object_path,
-                                                         self.local.type_path):
+                                                         self.local.type_path,
+                                                         self.local.object_marker_name):
             if cdist_object.cdist_type.is_install:
                 self.log.debug("Running in config mode, ignoring install object: {0}".format(cdist_object))
             else:
@@ -229,12 +230,12 @@ class Config(object):
                 for requirement in cdist_object.requirements_unfinished(cdist_object.autorequire):
                     autorequire_names.append(requirement.name)
 
-                requirements = ", ".join(requirement_names)
-                autorequire  = ", ".join(autorequire_names)
-                info_string.append("%s requires: %s autorequires: %s" % (cdist_object.name, requirements, autorequire))
+                requirements = "\n        ".join(requirement_names)
+                autorequire  = "\n        ".join(autorequire_names)
+                info_string.append("%s requires:\n        %s\n%s autorequires:\n        %s" % (cdist_object.name, requirements, cdist_object.name, autorequire))
 
-            raise cdist.UnresolvableRequirementsError("The requirements of the following objects could not be resolved: %s" %
-                ("; ".join(info_string)))
+            raise cdist.UnresolvableRequirementsError("The requirements of the following objects could not be resolved:\n%s" %
+                ("\n".join(info_string)))
 
     def object_prepare(self, cdist_object):
         """Prepare object: Run type explorer + manifest"""
