@@ -23,6 +23,7 @@
 import io
 import os
 import sys
+import glob
 import subprocess
 import logging
 
@@ -94,9 +95,17 @@ class Remote(object):
         """Transfer a file or directory to the remote side."""
         self.log.debug("Remote transfer: %s -> %s", source, destination)
         self.rmdir(destination)
-        command = self._copy.split()
-        command.extend(["-r", source, self.target_host + ":" + destination])
-        self._run_command(command)
+        if os.path.isdir(source):
+            self.mkdir(destination)
+            for f in glob.glob1(source, '*'):
+                command = self._copy.split()
+                path = os.path.join(source, f)
+                command.extend([path, '{0}:{1}'.format(self.target_host, destination)])
+                self._run_command(command)
+        else:
+            command = self._copy.split()
+            command.extend([source, '{0}:{1}'.format(self.target_host, destination)])
+            self._run_command(command)
 
     def run_script(self, script, env=None, return_output=False):
         """Run the given script with the given environment on the remote side.
@@ -104,7 +113,7 @@ class Remote(object):
 
         """
 
-        command = ["/bin/sh", "-e"]
+        command = [ os.environ.get('CDIST_REMOTE_SHELL',"/bin/sh") , "-e"]
         command.append(script)
 
         return self.run(command, env, return_output)
@@ -121,12 +130,28 @@ class Remote(object):
         # FIXME: replace this by -o SendEnv name -o SendEnv name ... to ssh?
         # can't pass environment to remote side, so prepend command with
         # variable declarations
+
+        # cdist command prepended with variable assignments expects
+        # posix shell (bourne, bash) at the remote as user default shell.
+        # If remote user shell isn't poxis shell, but for e.g. csh/tcsh
+        # then these var assignments are not var assignments for this
+        # remote shell, it tries to execute it as a command and fails.
+        # So really do this by default:
+        # /bin/sh -c 'export <var assignments>; command'
+        # so that constructed remote command isn't dependent on remote
+        # shell. Do this only if env is not None. env breaks this.
+        # Explicitly use /bin/sh, because var assignments assume poxis
+        # shell already.
+        # This leaves the posibility to write script that needs to be run
+        # remotely in e.g. csh and setting up CDIST_REMOTE_SHELL to e.g.
+        # /bin/csh will execute this script in the right way.
         if env:
-            remote_env = ["%s=%s" % item for item in env.items()]
-            cmd.extend(remote_env)
-
-        cmd.extend(command)
-
+            remote_env = [" export %s=%s;" % item for item in env.items()]
+            string_cmd = ("/bin/sh -c '" + " ".join(remote_env)
+                + " ".join(command) + "'")
+            cmd.append(string_cmd)
+        else:
+            cmd.extend(command)
         return self._run_command(cmd, env=env, return_output=return_output)
 
     def _run_command(self, command, env=None, return_output=False):
